# -*- coding: utf-8 -*-

from Model.Except import NotImplementedMethodeError

from Model.BoundaryCondition.BoundaryCondition import BoundaryCondition


class NotDefined(BoundaryCondition):
    def __init__(self, id:int = -1, name:str = "", status = None):
        super(NotDefined, self).__init__(id=id, name=name, status=status)

        self._type = "ND"
        self._header = ["x", "y"]

    @property
    def _default_0(self):
        return 0.0

class PonctualContribution(BoundaryCondition):
    def __init__(self, id:int = -1, name:str = "", status = None):
        super(PonctualContribution, self).__init__(id=id, name=name, status=status)

        self._type = "PC"
        self._header = ["time", "discharge"]
        self._types = [PonctualContribution.time_convert, float]

    @classmethod
    def compatibility(cls):
        return ["liquid"]

class TimeOverZ(BoundaryCondition):
    def __init__(self, id:int = -1, name:str = "", status = None):
        super(TimeOverZ, self).__init__(id=id, name=name, status=status)

        self._type = "TZ"
        self._header = ["time", "z"]
        self._types = [TimeOverZ.time_convert, float]

    @classmethod
    def compatibility(cls):
        return ["liquid"]

class TimeOverDischarge(BoundaryCondition):
    def __init__(self, id:int = -1, name:str = "", status = None):
        super(TimeOverDischarge, self).__init__(id=id, name=name, status=status)

        self._type = "TD"
        self._header = ["time", "discharge"]
        self._types = [TimeOverDischarge.time_convert, float]

    @classmethod
    def compatibility(cls):
        return ["liquid"]

class ZOverDischarge(BoundaryCondition):
    def __init__(self, id:int = -1, name:str = "", status = None):
        super(ZOverDischarge, self).__init__(id=id, name=name, status=status)

        self._type = "ZD"
        self._header = ["z", "discharge"]
        self._types = [float, float]

    @classmethod
    def compatibility(cls):
        return ["liquid"]

    @property
    def _default_0(self):
        return 0.0
