# -*- coding: utf-8 -*-

from tools import trace, timer

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtGui import (
    QColor, QBrush,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox,
)

_translate = QCoreApplication.translate


class TableModel(QAbstractTableModel):
    def __init__(self, data=None):
        super(QAbstractTableModel, self).__init__()
        self._headers = ["name", "status"]
        self._data = data

    def flags(self, index):
        options = Qt.ItemIsEnabled | Qt.ItemIsSelectable
        return options

    def rowCount(self, parent):
        return len(self._data)

    def columnCount(self, parent):
        return len(self._headers)

    def data(self, index, role):
        row = index.row()
        column = index.column()

        if role == Qt.ForegroundRole:
            if self._headers[column] == "status":
                color = Qt.gray

                if self._data[row].is_ok():
                    color = Qt.green
                elif self._data[row].is_warning():
                    color = Qt.yellow
                elif self._data[row].is_error():
                    color = Qt.red

                return QBrush(color)
            return QVariant()

        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        if self._headers[column] == "name":
            return self._data[row].name
        if self._headers[column] == "status":
            return self._data[row].summary

        return QVariant()

    def headerData(self, section, orientation, role):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return self._headers[section]

        return QVariant()

    def update(self):
        self.layoutChanged.emit()
