# -*- coding: utf-8 -*-

from tools import trace, timer

from PyQt5.QtCore import (
    Qt, QVariant, QAbstractTableModel,
    QCoreApplication, QModelIndex, pyqtSlot,
    QRect,
)

from PyQt5.QtWidgets import (
    QDialogButtonBox, QPushButton, QLineEdit,
    QFileDialog, QTableView, QAbstractItemView,
    QUndoStack, QShortcut, QAction, QItemDelegate,
    QComboBox,
)

from View.SolverParameters.UndoCommand import *
from View.SolverParameters import translate as tr

from Solver.Solvers import solver_long_name, solver_type_list

_translate = QCoreApplication.translate

class TableModel(QAbstractTableModel):
    def __init__(self, data=None, undo=None, tab=""):
        super(QAbstractTableModel, self).__init__()
        self._headers = list(tr.table_headers.keys())
        self._data = data
        self._undo = undo
        self._tab = tab
        self._params = self._data.get_params(self._tab)

    def flags(self, index):
        options = Qt.ItemIsEnabled | Qt.ItemIsSelectable

        column = index.column()

        if self._headers[column] == "value":
            options |= Qt.ItemIsEditable

        return options

    def rowCount(self, parent):
        return len(self._params)

    def columnCount(self, parent):
        return len(self._headers)

    def data(self, index, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return QVariant()

        row = index.row()
        column = index.column()
        cname = self._headers[column]

        if cname == "name":
            value = self._params.get(row)[cname]
            if value in tr.names:
                value = tr.names[value]
            return value
        elif cname == "value":
            value = self._params.get(row)[cname]
            if value in tr.yes_no:
                value = tr.yes_no[value]
            return value

        return QVariant()

    def headerData(self, section, orientation, role):
        if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal:
            return tr.table_headers[self._headers[section]]

        return QVariant()

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid() or role != Qt.EditRole:
            return False

        row = index.row()
        column = index.column()

        if self._headers[column] == "value":
            if value in tr.r_yes_no:
                value = tr.r_yes_no[value].lower()

            self._undo.push(
                SetCommand(
                    self._params, row,
                    "value", value
                )
            )

        self.dataChanged.emit(index, index)
        return True

    # def add(self, row, parent=QModelIndex()):
    #     self.beginInsertRows(parent, row, row - 1)

    #     self._undo.push(
    #         AddCommand(
    #             self._params, row
    #         )
    #     )

    #     self.endInsertRows()
    #     self.layoutChanged.emit()

    # def delete(self, rows, parent=QModelIndex()):
    #     self.beginRemoveRows(parent, rows[0], rows[-1])

    #     self._undo.push(
    #         DelCommand(
    #             self._params, rows
    #         )
    #     )

    #     self.endRemoveRows()
    #     self.layoutChanged.emit()

    # def sort(self, _reverse, parent=QModelIndex()):
    #     self.layoutAboutToBeChanged.emit()

    #     self._undo.push(
    #         SortCommand(
    #             self._params, False
    #         )
    #     )

    #     self.layoutAboutToBeChanged.emit()
    #     self.layoutChanged.emit()

    # def move_up(self, row, parent=QModelIndex()):
    #     if row <= 0:
    #         return

    #     target = row + 2

    #     self.beginMoveRows(parent, row - 1, row - 1, parent, target)

    #     self._undo_stack.push(
    #         MoveCommand(
    #             self._params, "up", row
    #         )
    #     )

    #     self.endMoveRows()
    #     self.layoutChanged.emit()

    # def move_down(self, index, parent=QModelIndex()):
    #     if row > len(self._params):
    #         return

    #     target = row

    #     self.beginMoveRows(parent, row + 1, row + 1, parent, target)

    #     self._undo_stack.push(
    #         MoveCommand(
    #             self._params, "down", row
    #         )
    #     )

    #     self.endMoveRows()
    #     self.layoutChanged.emit()

    def undo(self):
        self._undo.undo()
        self.layoutChanged.emit()

    def redo(self):
        self._undo.redo()
        self.layoutChanged.emit()
