#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys, os
import locale
import logging

from PyQt5.QtCore import QTranslator
from PyQt5.QtWidgets import QApplication

from config import Config
from tools import (
    reset_timers, display_timers, timer
)

from View.MainWindow import ApplicationWindow
from Model.Study import Study

logging.basicConfig(
    level=logging.DEBUG,
    format='[PAMHYR][%(levelname)s] %(message)s'
)
logger = logging.getLogger()
logger.setLevel(logging.INFO)

def main():
    conf = Config.load()
    app = QApplication(sys.argv)

    translator = QTranslator()

    lang_file = ""
    if conf.lang == "":
        # System language
        sys_lang = locale.getdefaultlocale()
        if "fr" in sys_lang[0]:
            lang_file = os.path.dirname(__file__) + "/lang/fr.qm"
    elif conf.lang == "fr":
        # French
        lang_file = os.path.dirname(__file__) + "/lang/fr.qm"
    else:
        # English default language
        lang_file = ""

    if lang_file != "":
        ok = translator.load(lang_file)
        if not ok:
            logger.error("Failed to load translate file")

    app.installTranslator(translator)

    application = ApplicationWindow(conf=conf)
    application.show()

    ret = app.exec_()
    display_timers()
    sys.exit(ret)

if __name__ == "__main__":
    main()
